package de.fhdw.wtf.tooling.builders;

import java.nio.file.Paths;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.IWordDetector;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.exception.editor.MultipleCheckExceptions;
import de.fhdw.wtf.tooling.SyntaxCheck;
import de.fhdw.wtf.tooling.builders.result.ExecutionFailedResult;
import de.fhdw.wtf.tooling.builders.result.ExecutionSuccessfulResult;
import de.fhdw.wtf.tooling.editor.ColorManager;
import de.fhdw.wtf.tooling.editor.ConsoleWriter;
import de.fhdw.wtf.tooling.editor.WTFColorConstants;
import de.fhdw.wtf.tooling.editor.WTFRuleScanner;
import de.fhdw.wtf.tooling.editor.proposals.AttributeModifierProposal;
import de.fhdw.wtf.tooling.editor.proposals.UserTypeProposal;

public class CheckCommand extends IntegrationCommand {
	
	public CheckCommand(final IProject project) {
		super(project);
	}
	
	public void checkModel() {
		final Collection<IResource> modelResources = this.findModelResources();
		if (modelResources.size() == 1) {
			final IResource resource = modelResources.iterator().next();
			this.clear(resource);
			this.checkModelResource(resource);
		} else {
			this.setResult(new ExecutionFailedResult<Model>());
			ConsoleWriter.print("More than one model file found, this feature is not yet supported");
		}
	}
	
	private void checkModelResource(final IResource modelResource) {
		
		final SyntaxCheck modelManager = SyntaxCheck.getInstance();
		try {
			
			final String fileContent =
					new String(java.nio.file.Files.readAllBytes(Paths.get(modelResource.getLocationURI())));
			
			final Model model = modelManager.getModelFromString(fileContent);
			this.setResult(new ExecutionSuccessfulResult<>(model));
			this.updateUserTypes();
		} catch (final MultipleCheckExceptions e) {
			this.reactOnExceptions(modelResource, e);
			this.setResult(new ExecutionFailedResult<Model>());
		} catch (final Exception e) {
			this.reactOnUnknownException(modelResource, e);
			this.setResult(new ExecutionFailedResult<Model>());
		}
	}
	
	private void updateUserTypes() {
		final ExecutionSuccessfulResult<Model> result = (ExecutionSuccessfulResult<Model>) this.getResult();
		
		final UserTypeProposal proposal = UserTypeProposal.create();
		final int length = proposal.PROPOSALS.length;
		final ColorManager colorManager = ColorManager.create();
		WTFRuleScanner.create(colorManager).generateStandardRules();
		
		final AttributeModifierProposal attributeModifierProposal = AttributeModifierProposal.create();
		attributeModifierProposal.generateStandardPredecessors();
		
		for (int i = 0; i < length; i++) {
			if (proposal.PROPOSALS[i] == null) {
				break;
			} else {
				proposal.PROPOSALS[i] = null;
			}
		}
		
		final Iterator<ClassType> iterator = result.getResult().calcAllGroupsAndClasses().getFirst().iterator();
		final Vector<String> newProposals = new Vector<>();
		while (iterator.hasNext()) {
			final ClassType current = iterator.next();
			System.out.println(current.getTypeName().toString());
			
			final String proposalStr = current.getTypeName().getLastAddedName().toString();
			if (proposal.PROPOSALS[0] == null) {
				newProposals.add(proposalStr);
				proposal.PROPOSALS[0] = proposalStr;
			} else {
				for (int i = 0; i < length; i++) {
					final String currentProposal = proposal.PROPOSALS[i];
					if (currentProposal != null) {
						if (currentProposal.equals(proposalStr)) {
							break;
						}
					} else {
						proposal.PROPOSALS[i] = proposalStr;
						newProposals.add(proposalStr);
						AttributeModifierProposal.PREDECESSORS.add(proposalStr);
						break;
					}
				}
			}
		}
		for (final String str : newProposals) {
			final IToken other = new Token(new TextAttribute(ColorManager.create().getColor(WTFColorConstants.OTHER)));
			final IToken userType =
					new Token(new TextAttribute(ColorManager.create().getColor(WTFColorConstants.OTHER)));
			final char firstChar = str.charAt(0);
			final WordRule userTypeRule = new WordRule(new IWordDetector() {
				@Override
				public boolean isWordStart(final char c) {
					return c == firstChar;
				}
				
				@Override
				public boolean isWordPart(final char c) {
					return !(Character.isWhitespace(c) || c == ',' || c == ';');
				}
			}, other);
			userTypeRule.addWord(str, userType);
			WTFRuleScanner.create(ColorManager.create()).updateRules(userTypeRule);
		}
	}
	
	@Override
	public void execute() {
		this.checkModel();
	}
}
